/**
 * menu-container.component
 */

import {
    Component, OnInit, Input, trigger, state, style, transition, animate, HostListener,
    Output, EventEmitter
} from '@angular/core';
import { MenuOptions, IMenuConfig, IMenuWing, IMenuButton } from './menu-options.service';

/*
declare let require: any;
const myDpStyles: string = require("./menu-container.component.scss");
const myDpTpl: string = require("./menu-container.component.html");
*/

@Component({
    selector: 'app-menu-container',
    template: `<div class="menu-container" [ngStyle]="menuContainerStyle"><button class="menu-btn" [ngStyle]="menuBtnStyle" (tap)="toggleMenu()" (panstart)="onPanStart()" (panend)="onPanEnd()"><span [@menuScaleInOut]="menuState.toString()" (@menuScaleInOut.done)="animationDone($event)">Menu </span><span class="btn-cross" [@crossScaleInOut]="menuState.toString()" (@crossScaleInOut.done)="animationDone($event)"><svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" id="cross" class="img" x="0px" y="0px" viewBox="0 0 212.982 212.982" style="enable-background:new 0 0 212.982 212.982" xml:space="preserve" [attr.width]="menuOptions.Button.crossImgSize" [attr.height]="menuOptions.Button.crossImgSize"><g id="Close"><path style="fill-rule:evenodd;clip-rule:evenodd" [attr.fill]="menuOptions.Button.color" d="M131.804,106.491l75.936-75.936c6.99-6.99,6.99-18.323,0-25.312   c-6.99-6.99-18.322-6.99-25.312,0l-75.937,75.937L30.554,5.242c-6.99-6.99-18.322-6.99-25.312,0c-6.989,6.99-6.989,18.323,0,25.312   l75.937,75.936L5.242,182.427c-6.989,6.99-6.989,18.323,0,25.312c6.99,6.99,18.322,6.99,25.312,0l75.937-75.937l75.937,75.937   c6.989,6.99,18.322,6.99,25.312,0c6.99-6.99,6.99-18.322,0-25.312L131.804,106.491z"/></g></svg></span></button><div class="menu-list" [ngStyle]="menuListStyle"><app-menu-wing *ngFor="let wing of wings; let i = index" [textAnchor]="textAnchor" [textRotate]="textRotate" [position]="positionClass" [wing]="wing" [svgPath]="svgPath" [index]="i" [menuState]="menuState" (wingClicked)="clickWing($event)"></app-menu-wing></div></div>`,
    styles: [`.menu-container{position:fixed;z-index:99999}.menu-btn{-moz-box-shadow:-2px 6px 12px #8e8e8e;box-shadow:-2px 6px 12px #8e8e8e;outline:0;position:absolute;border:none;z-index:1000;cursor:pointer;-moz-border-radius:100%;border-radius:100%;background-color:#ff7f7f;color:#fff;font-size:14px}.menu-btn .img{position:absolute;top:0;left:0;right:0;bottom:0;margin:auto;pointer-events:none;vertical-align:middle;display:block}.menu-btn span{display:block;vertical-align:middle;pointer-events:none}.menu-btn .btn-cross{position:absolute;top:0;left:0;right:0;bottom:0;margin:auto}.menu-list{pointer-events:none;position:absolute}`],
    animations: [
        trigger('menuScaleInOut', [
            state('false', style({
                opacity: 1,
                transform: 'scale(1)'
            })),
            state('true', style({
                opacity: 0,
                transform: 'scale(0)'
            })),
            transition('false <=> true', animate('300ms linear'))
        ]),
        trigger('crossScaleInOut', [
            state('true', style({
                opacity: 1,
                transform: 'scale(1)'
            })),
            state('false', style({
                opacity: 0,
                transform: 'scale(0)'
            })),
            transition('false <=> true', animate('300ms linear'))
        ])
    ],
})
export class MenuContainerComponent implements OnInit {

    @Input() private options: IMenuConfig;

    @Input() private button: IMenuButton;

    // The space between the menu and the boundaries of the page window
    @Input() private gutter: Object;

    @Input() private wings: IMenuWing[];

    @Input() private startAngles: Object; // the first wing starting angle

    @Output() private onWingSelected = new EventEmitter<IMenuWing>();

    private menuContainerStyle: Object;
    private menuBtnStyle: Object;
    private menuListStyle: Object;
    private allowTransition: boolean = true; // a flag to indicate if button text animation finished
    private dragStart: boolean = false; // A flag to indicate the drag move begins
    private svgPath: string;
    private menuState: boolean; // A flag to indicate if the menu is open
    private positionClass: string; // menu's position
    private textRotate: number;
    private textAnchor: string;

    constructor( public menuOptions: MenuOptions ) {
    }

    public ngOnInit() {
        this.menuOptions.setMenuOptions(this.options, this.button, this.gutter, this.startAngles);
        this.menuState = this.menuOptions.MenuConfig.defaultOpen;
        this.positionClass = this.menuOptions.MenuConfig.defaultPosition;
        this.setElementsStyle();
        this.calculateSvgPath();
        this.calculateMenuContainerPosition();
    }

    public animationDone() {
        this.allowTransition = true;
    }

    public clickWing( wing: IMenuWing ): void {
        this.onWingSelected.emit(wing);
    }

    public toggleMenu() {
        if (this.allowTransition) {
            this.menuState = !this.menuState;
            this.allowTransition = false;
        }
    }

    public onPanStart(): void {
        this.dragStart = true;
        this.menuContainerStyle['transition'] = 'none';
        this.menuContainerStyle['-webkit-transition'] = 'none';
        this.menuContainerStyle['-moz-transition'] = 'none';
        this.menuContainerStyle['-ms-transition'] = 'none';
    }

    public onPanEnd(): void {
        this.dragStart = false;
        this.menuContainerStyle['transition'] = 'all 900ms cubic-bezier(0.680, -0.550, 0.265, 1.550)';
        this.menuContainerStyle['-webkit-transition'] = 'all 900ms cubic-bezier(0.680, -0.550, 0.265, 1.550)';
        this.menuContainerStyle['-moz-transition'] = 'all 900ms cubic-bezier(0.680, -0.550, 0.265, 1.550)';
        this.menuContainerStyle['-ms-transition'] = 'all 900ms cubic-bezier(0.680, -0.550, 0.265, 1.550)';

        let centreX = window.innerWidth / 2 -
            this.menuOptions.Button.width / 2;
        let centreY = window.innerHeight / 2 -
            this.menuOptions.Button.width / 2;

        if (this.menuContainerStyle['top.px'] > centreY &&
            this.menuContainerStyle['left.px'] < centreX) {
            this.positionClass = 'bottomLeft';
            this.textRotate = 0;
            this.textAnchor = 'middle';
        } else if (this.menuContainerStyle['top.px'] < centreY &&
            this.menuContainerStyle['left.px'] < centreX) {
            this.positionClass = 'topLeft';
            this.textRotate = 0;
            this.textAnchor = 'middle';
        } else if (this.menuContainerStyle['top.px'] < centreY &&
            this.menuContainerStyle['left.px'] > centreX) {
            this.positionClass = 'topRight';
            this.textRotate = 180;
            this.textAnchor = 'end';
        } else if (this.menuContainerStyle['top.px'] > centreY &&
            this.menuContainerStyle['left.px'] > centreX) {
            this.positionClass = 'bottomRight';
            this.textRotate = 180;
            this.textAnchor = 'end';
        }
        this.calculateMenuContainerPosition();
    }

    @HostListener('document:panmove', ['$event'])
    public onMenuMove( event: any ): void {
        if (this.dragStart) {
            let y = event.center.y;
            let x = event.center.x;
            this.menuContainerStyle['top.px'] = y - this.menuOptions.Button.width / 2;
            this.menuContainerStyle['left.px'] = x - this.menuOptions.Button.width / 2;
        }
    }

    private calculateMenuContainerPosition() {
        if (this.positionClass === 'topLeft') {
            let top = this.menuOptions.Gutter.top;
            let left = this.menuOptions.Gutter.left;
            this.menuContainerStyle['top.px'] = top;
            this.menuContainerStyle['left.px'] = left;
            this.textAnchor = 'middle';
            this.textRotate = 0;
            this.menuOptions.Center = {x: left, y: top};

        } else if (this.positionClass === 'topRight') {
            let top = this.menuOptions.Gutter.top;
            let left = window.innerWidth - this.menuOptions.Button.width -
                this.menuOptions.Gutter.right;
            this.menuContainerStyle['top.px'] = top;
            this.menuContainerStyle['left.px'] = left;
            this.textAnchor = 'end';
            this.textRotate = 180;
            this.menuOptions.Center = {x: left, y: top};

        } else if (this.positionClass === 'bottomLeft') {
            let top = window.innerHeight - this.menuOptions.Button.width -
                this.menuOptions.Gutter.bottom;
            let left = this.menuOptions.Gutter.left;
            this.menuContainerStyle['top.px'] = top;
            this.menuContainerStyle['left.px'] = left;
            this.textAnchor = 'middle';
            this.textRotate = 0;
            this.menuOptions.Center = {x: left, y: top};

        } else if (this.positionClass === 'bottomRight') {

            let top = window.innerHeight - this.menuOptions.Button.width
                - this.menuOptions.Gutter.bottom;
            let left = window.innerWidth - this.menuOptions.Button.width
                - this.menuOptions.Gutter.right;
            this.menuContainerStyle['top.px'] = top;
            this.menuContainerStyle['left.px'] = left;
            this.textAnchor = 'end';
            this.textRotate = 180;
            this.menuOptions.Center = {x: left, y: top};
        }
    }

    private setElementsStyle(): void {
        this.menuContainerStyle = {
            'font-family': this.menuOptions.MenuConfig.font,
            'width.px': this.menuOptions.Button.width,
            'height.px': this.menuOptions.Button.width,
            'top.px': 0,
            'left.px': 0,
            'transition': 'none',
            '-webkit-transition': 'none',
            '-ms-transition': 'none',
            '-moz-transition': 'none',
        };
        this.menuBtnStyle = {
            'width.px': this.menuOptions.Button.width,
            'height.px': this.menuOptions.Button.width,
            'background': this.menuOptions.Button.backgroundColor,
            'color': this.menuOptions.Button.color,
            'font-size': this.menuOptions.Button.fontSize,
            'font-weight': this.menuOptions.Button.fontWeight,
        };
        this.menuListStyle = {
            'top.px': -(this.menuOptions.MenuConfig.radius - this.menuOptions.Button.width) / 2,
            'left.px': this.menuOptions.Button.width / 2,
            'width.px': this.menuOptions.MenuConfig.radius,
            'height.px': this.menuOptions.MenuConfig.radius,
        };
    }

    private calculateSvgPath() {
        let buttonWidth = this.menuOptions.Button.width;
        let offset = this.menuOptions.MenuConfig.offset;
        let angle = this.menuOptions.MenuConfig.angle;
        let radius = this.menuOptions.MenuConfig.radius;
        let innerRadius = buttonWidth / 2 + offset;
        let x1 = Math.floor(radius * Math.cos(Math.PI * (360 - angle / 2) / 180));
        let y1 = Math.floor(radius / 2 + radius * Math.sin(Math.PI * (360 - angle / 2) / 180));
        let x2 = Math.floor(radius * Math.cos(Math.PI * (angle / 2) / 180));
        let y2 = Math.floor(radius / 2 + radius * Math.sin(Math.PI * (angle / 2) / 180));
        let a1 = Math.floor(innerRadius * Math.cos(Math.PI * (360 - angle / 2) / 180));
        let b1 = Math.floor(radius / 2 + innerRadius * Math.sin(Math.PI * (360 - angle / 2) / 180));
        let a2 = Math.floor(innerRadius * Math.cos(Math.PI * (angle / 2) / 180));
        let b2 = Math.floor(radius / 2 + 1 + innerRadius * Math.sin(Math.PI * (angle / 2) / 180));

        this.svgPath = 'M' + a1 + ',' + b1 + ' L' + x1 + ',' + y1 + ' A' +
            radius + ',' + radius + ' 0 0, 1' + ' ' + x2 + ',' + y2 +
            ' L' + a2 + ',' + b2 + '  A' + innerRadius + ',' + innerRadius +
            ' 1 0, 0' + ' ' + a1 + ',' + b1 + ' z';
    }
}
