/**
 * menu-wing.component
 */

import { Component, OnInit, Input, OnChanges, SimpleChanges, OnDestroy, Output, EventEmitter } from '@angular/core';
import { IMenuWing, MenuOptions } from './menu-options.service';
import { SpinService } from './menu-spin.service';

/*
declare let require: any;
const myDpStyles: string = require("./menu-wing.component.scss");
const myDpTpl: string = require("./menu-wing.component.html");
*/

@Component({
    selector: 'app-menu-wing',
    template: `<div class="wing wing-{{index}}" [ngStyle]="{
        'transition': spinService.TransitionStyle,
        '-webkit-transition': spinService.TransitionStyle,
        '-moz-transition': spinService.TransitionStyle,
        '-ms-transition': spinService.TransitionStyle,
        'transform': 'rotate(' + (rotateDeg + spinService.SpinDegrees) + 'deg) '+ 'scale(' + scaleSize +')',
        '-webkit-transform': 'rotate(' + (rotateDeg + spinService.SpinDegrees) + 'deg) '+ 'scale(' + scaleSize +')',
        '-ms-transform': 'rotate(' + (rotateDeg + spinService.SpinDegrees) + 'deg) '+ 'scale(' + scaleSize +')',
        '-moz-transform': 'rotate(' + (rotateDeg + spinService.SpinDegrees) + 'deg) '+ 'scale(' + scaleSize +')',
        '-o-transform': 'rotate(' + (rotateDeg + spinService.SpinDegrees) + 'deg) '+ 'scale(' + scaleSize +')'
     }"><svg class="wing-svg" width="100%" height="100%"><path style="pointer-events:auto" (tap)="onClick()" (panstart)="onPanStart($event)" (panmove)="onRotate($event)" (panend)="onPanEnd($event)" (mouseover)="onMouseOver()" (mouseout)="onMouseOut()" [attr.fill]="wing.color" [attr.d]="svgPath"></path><text class="wing-text" *ngIf="!menuOptions.MenuConfig.onlyIcons" x="50%" y="50%" dominant-baseline="central" style="text-shadow: 1px 1px 1px rgba(0, 0, 0, 0.2)" [ngStyle]="{'font-size': menuOptions.MenuConfig.wingFontSize + 'px'}" [attr.text-anchor]="textAnchor" [attr.transform]="'rotate('+ textRotate + ', ' + menuOptions.MenuConfig.radius/2  + ', ' + menuOptions.MenuConfig.radius/2 + ')'" [attr.fill]="wing.titleColor || menuOptions.MenuConfig.wingFontColor">{{wing.title}}</text></svg> <i class="{{wing.icon.name}}" *ngIf="menuOptions.MenuConfig.showIcons || menuOptions.MenuConfig.onlyIcons" [ngStyle]="{
            'color': wing.icon.color || menuOptions.MenuConfig.wingFontColor,
            'font-size': iconSize + 'px',
            'width': iconSize + 'px',
            'height': iconSize + 'px',
            'transform': 'translate(' + iconX + 'px, ' + iconY + 'px) rotate('+ (rotateDeg + spinService.SpinDegrees) * -1 + 'deg)',
            '-webkit-transform': 'translate(' + iconX + 'px, ' + iconY + 'px) rotate('+ (rotateDeg + spinService.SpinDegrees) * -1 + 'deg)',
            '-ms-transform': 'translate(' + iconX + 'px, ' + iconY + 'px) rotate('+ (rotateDeg + spinService.SpinDegrees) * -1 + 'deg)',
            '-moz-transform': 'translate(' + iconX + 'px, ' + iconY + 'px) rotate('+ (rotateDeg + spinService.SpinDegrees) * -1 + 'deg)',
            '-o-transform': 'translate(' + iconX + 'px, ' + iconY + 'px) rotate('+ (rotateDeg + spinService.SpinDegrees) * -1 + 'deg)'
       }"></i></div>`,
    styles: [`.wing{-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;pointer-events:none;width:100%;height:100%;position:absolute;cursor:pointer;-webkit-transform-origin:0 50%;-moz-transform-origin:0 50%;-ms-transform-origin:0 50%;transform-origin:0 50%;-webkit-transition:all .3s cubic-bezier(.68,-.55,.265,1.55);-moz-transition:all .3s cubic-bezier(.68,-.55,.265,1.55);transition:all .3s cubic-bezier(.68,-.55,.265,1.55)}.wing i{text-align:center;-webkit-transform-origin:50% 50%;-moz-transform-origin:50% 50%;-ms-transform-origin:50% 50%;transform-origin:50% 50%}`],
})
export class MenuWingComponent implements OnInit, OnChanges, OnDestroy {

    @Input() wing: IMenuWing;
    @Input() index: number;
    @Input() svgPath: string;
    @Input() menuState: boolean;
    @Input() position: string;
    @Input() textRotate: number;
    @Input() textAnchor: string;
    @Output() wingClicked = new EventEmitter<IMenuWing>();

    private timeOutId: number = 0;
    private scaleSize: number;
    private rotateDeg: number;
    private iconX: number;
    private iconY: number;
    private iconSize: number;

    constructor( private menuOptions: MenuOptions,
                 private spinService: SpinService ) {
    }

    public ngOnInit() {
        this.iconSize = this.wing.icon.size || this.menuOptions.MenuConfig.wingIconSize;
        if (this.menuOptions.MenuConfig.onlyIcons) {
            this.iconX = this.menuOptions.MenuConfig.radius - this.menuOptions.MenuConfig.radius / 2 + this.iconSize / 4;
        } else {
            this.iconX = this.menuOptions.MenuConfig.radius - this.iconSize - 8;
        }
        this.iconY = -(this.menuOptions.MenuConfig.radius / 2 + this.iconSize / 2 + 5);
    }

    public ngOnDestroy(): void {
        this.clearTimer();
    }

    public ngOnChanges( changes: SimpleChanges ): void {

        if (changes['position'] && changes['position'].isFirstChange()) {
            this.rotateDeg = this.menuOptions.StartAngles[this.position];
            this.scaleSize = 0;
        }

        if (changes['position'] && !changes['position'].isFirstChange() &&
            this.scaleSize > 0) {
            this.rotateDeg = this.menuOptions.StartAngles[this.position]
                + (this.index * this.menuOptions.MenuConfig.angle);
        }

        if (changes['menuState'] &&
            changes['menuState'].previousValue === true &&
            changes['menuState'].currentValue === false) {
            this.clearTimer();
            this.rotateDeg = this.menuOptions.StartAngles[this.position];
            this.scaleSize = 1;

            this.timeOutId = window.setTimeout(() => {
                this.scaleSize = 0;
            }, 400);
        }

        if (changes['menuState'] && changes['menuState'].currentValue === true &&
            (changes['menuState'].previousValue === false || changes['menuState'].isFirstChange())) {
            this.clearTimer();
            this.rotateDeg = this.menuOptions.StartAngles[this.position];
            this.scaleSize = 1;

            this.timeOutId = window.setTimeout(() => {
                this.rotateDeg = this.menuOptions.StartAngles[this.position] +
                    (this.index * this.menuOptions.MenuConfig.angle);
            }, 400);
        }
    }

    public onMouseOver(): void {
        if (this.menuState) {
            this.scaleSize = 1.2;
        }
    }

    public onMouseOut(): void {
        if (this.menuState) {
            this.scaleSize = 1;
        }
    }

    public onClick(): void {
        this.wingClicked.emit(this.wing);
    }

    public onPanStart( event: any ): void {
        if (this.menuOptions.MenuConfig.spinable) {
            this.scaleSize = 1;
            this.spinService.setStartPosition(event.center);
        }
    }

    public onRotate( event: any ): void {
        if (this.menuOptions.MenuConfig.spinable) {
            this.spinService.setSpinDegrees(event.center);
        }
    }

    public onPanEnd( event: any ): void {
        if (this.menuOptions.MenuConfig.spinable) {
            this.spinService.setLastSpinDegrees(event.center);
        }
    }

    private clearTimer(): void {
        clearTimeout(this.timeOutId);
    }
}
